{% load i18n %}
<footer class="grp-module grp-submit-row grp-fixed-footer">
    <ul class="submit-row">
        {% if show_delete_link %}<li class="grp-float-left delete-link-container"><a href="delete/" class="grp-button grp-delete-link">{% trans "Delete" %}</a></li>{% endif %}

        {% comment %}
        This is the custom bit to support FSM based transitions. Iterate over the
        available transitions and add appropriate values that are then detected
        in project_utils.view_mixins.FSMTransitionMixin.
        {% endcomment %}
        {% for transition_name, transition in transitions %}
        <li class="grp-float-left submit-button-container"><input type="submit" value="{{ transition_name }}" class="default transition-{{ transition }}" name="_fsmtransition-{{ transition }}"/></li>
        {% endfor %}

        {% if show_save %}<li class="submit-button-container"><input type="submit" value="{% trans 'Save' %}" class="default" name="_save"/></li>{% endif %}
        {% if show_save_as_new %}<li class="submit-button-container"><input type="submit" value="{% trans 'Save as new' %}" name="_saveasnew"/></li>{% endif %}
        {% if show_save_and_add_another %}<li class="submit-button-container"><input type="submit" value="{% trans 'Save and add another' %}" name="_addanother"/></li>{% endif %}
        {% if show_save_and_continue %}<li class="submit-button-container"><input type="submit" value="{% trans 'Save and continue editing' %}" name="_continue"/></li>{% endif %}
    </ul><br clear="all" />
</footer>
